
# my-product-management-system

## Objective:
By the end of this session, students will be able to create class and functional components, style components using CSS, and perform CRUD operations in React using MockAPI. The project will be named "my-product-management-system" and will include pages to register categories and products.

## Agenda:
### 1. Introduction (30 minutes)
- Overview of React components: class components vs functional components.
- Introduction to React's component lifecycle methods.
- Brief introduction to CSS in React.
- Introduction to MockAPI for CRUD operations.

### 2. Setting Up the Project (30 minutes)
- Create a new React project using Create React App.
- Start the React development server.
- Brief overview of the project's folder structure.

\`\`\`bash
npx create-react-app my-product-management-system
cd my-product-management-system
npm start
\`\`\`

\`\`\`
my-product-management-system/
├── public/
│   ├── index.html
│   └── ...
├── src/
│   ├── components/
│   │   ├── Category/
│   │   │   ├── CategoryListClass.js
│   │   │   ├── CategoryFormClass.js
│   │   │   ├── CategoryListFunction.js
│   │   │   ├── CategoryFormFunction.js
│   │   │   ├── Category.css
│   │   ├── Product/
│   │   │   ├── ProductListClass.js
│   │   │   ├── ProductFormClass.js
│   │   │   ├── ProductListFunction.js
│   │   │   ├── ProductFormFunction.js
│   │   │   ├── Product.css
│   ├── services/
│   │   ├── ApiCategory.js
│   │   ├── ApiProduct.js
│   ├── App.js
│   ├── index.js
│   └── ...
├── package.json
└── ...
\`\`\`

### 3. Editing App.js (15 minutes)
- Modify \`App.js\` to include a welcome message and links to the category and product registration pages.

\`\`\`jsx
import React from 'react';
import { BrowserRouter as Router, Route, Link } from 'react-router-dom';
import CategoryFormClass from './components/Category/CategoryFormClass';
import ProductFormClass from './components/Product/ProductFormClass';

function App() {
  return (
    <Router>
      <div className="App">
        <h1>Product Management System</h1>
      </div>
    </Router>
  );
}

export default App;
\`\`\`

### 4. Creating Class Components (60 minutes)
- Deep dive into creating class components.
- Create `CategoryFormClass.js` and `CategoryListClass.js` as class components.
- Discuss state management, event handling, and lifecycle methods.

\`\`\`jsx
// CategoryFormClass.js
import React, { Component } from 'react';
import { createCategory } from '../../services/ApiCategory';

class CategoryFormClass extends Component {
  state = {
    name: '',
    description: ''
  };

  handleChange = (event) => {
    this.setState({ [event.target.name]: event.target.value });
  };

  handleSubmit = async (event) => {
    event.preventDefault();
    const { name, description } = this.state;
    await createCategory({ name, description });
    this.setState({ name: '', description: '' });
    this.props.fetchCategories();
  };

  render() {
    const { name, description } = this.state;

    return (
      <form onSubmit={this.handleSubmit}>
        <input
          type="text"
          name="name"
          value={name}
          onChange={this.handleChange}
          placeholder="Category Name"
        />
        <input
          type="text"
          name="description"
          value={description}
          onChange={this.handleChange}
          placeholder="Category Description"
        />
        <button type="submit">Add Category</button>
      </form>
    );
  }
}

export default CategoryFormClass;
\`\`\`

\`\`\`jsx
// CategoryListClass.js
import React, { Component } from 'react';
import CategoryFormClass from './CategoryFormClass';
import { getCategories, deleteCategory, updateCategory } from '../../services/ApiCategory';
import './Category.css';

class CategoryListClass extends Component {
  state = {
    categories: [],
    isEditing: false,
    currentCategory: { id: '', name: '', description: '' },
  };

  componentDidMount() {
    this.fetchCategories();
  }

  fetchCategories = async () => {
    const categories = await getCategories();
    this.setState({ categories });
  };

  handleDelete = async (id) => {
    await deleteCategory(id);
    this.fetchCategories();
  };

  handleEdit = (category) => {
    this.setState({ isEditing: true, currentCategory: category });
  };

  handleCancelEdit = () => {
    this.setState({ isEditing: false, currentCategory: { id: '', name: '', description: '' } });
  };

  handleChange = (event) => {
    const { name, value } = event.target;
    this.setState((prevState) => ({
      currentCategory: { ...prevState.currentCategory, [name]: value }
    }));
  };

  handleUpdate = async (event) => {
    event.preventDefault();
    const { currentCategory } = this.state;
    await updateCategory(currentCategory.id, { name: currentCategory.name, description: currentCategory.description });
    this.setState({ isEditing: false, currentCategory: { id: '', name: '', description: '' } });
    this.fetchCategories();
  };

  render() {
    const { categories, isEditing, currentCategory } = this.state;
    return (
      <div>
        <h2>Categories</h2>
        {isEditing ? (
          <form onSubmit={this.handleUpdate}>
            <input
              type="text"
              name="name"
              value={currentCategory.name}
              onChange={this.handleChange}
              placeholder="Category Name"
            />
            <input
              type="text"
              name="description"
              value={currentCategory.description}
              onChange={this.handleChange}
              placeholder="Category Description"
            />
            <button type="submit" className="button button-update">Update</button>
            <button type="button" className="button button-delete" onClick={this.handleCancelEdit}>Cancel</button>
          </form>
        ) : (
          <CategoryFormClass fetchCategories={this.fetchCategories} />
        )}
        <table>
          <thead>
            <tr>
              <th>ID</th>
              <th>Name</th>
              <th>Description</th>
              <th>Actions</th>
            </tr>
          </thead>
          <tbody>
            {categories.map(category => (
              <tr key={category.id}>
                <td>{category.id}</td>
                <td>{category.name}</td>
                <td>{category.description}</td>
                <td>
                  <button className="button button-update" onClick={() => this.handleEdit(category)}>Edit</button>
                  <button className="button button-delete" onClick={() => this.handleDelete(category.id)}>Delete</button>
                </td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>
    );
  }
}

export default CategoryListClass;
\`\`\`

### 5. Creating Functional Components (60 minutes)
- Deep dive into creating functional components with hooks.
- Create `CategoryFormFunction.js` and `CategoryListFunction.js` as functional components.
- Discuss useState and useEffect hooks.

\`\`\`jsx
// CategoryFormFunction.js
import React, { useState } from 'react';
import { createCategory } from '../../services/ApiCategory';

const CategoryFormFunction = ({ fetchCategories }) => {
  const [categoryName, setCategoryName] = useState('');
  const [description, setDescription] = useState('');

  const handleChange = (event) => {
    const { name, value } = event.target;
    if (name === 'name') setCategoryName(value);
    if (name === 'description') setDescription(value);
  };

  const handleSubmit = async (event) => {
    event.preventDefault();
    await createCategory({ name: categoryName, description });
    setCategoryName('');
    setDescription('');
    fetchCategories();
  };

  return (
    <form onSubmit={handleSubmit}>
      <input
        type="text"
        name="name"
        value={categoryName}
        onChange={handleChange}
        placeholder="Category Name"
      />
      <input
        type="text"
        name="description"
        value={description}
        onChange={handleChange}
        placeholder="Category Description"
      />
      <button type="submit">Add Category</button>
    </form>
  );
};

export default CategoryFormFunction;
\`\`\`

\`\`\`jsx
// CategoryListFunction.js
import React, { useEffect, useState } from 'react';
import { getCategories, deleteCategory, updateCategory } from '../../services/ApiCategory';
import './Category.css';
import CategoryFormFunction from './CategoryFormFunction';

const CategoryListFunction = () => {
  const [categories, setCategories] = useState([]);
  const [isEditing, setIsEditing] = useState(false);
  const [currentCategory, setCurrentCategory] = useState({ id: '', name: '', description: '' });

  useEffect(() => {
    fetchCategories();
  }, []);

  const fetchCategories = async () => {
    const categories = await getCategories();
    setCategories(categories);
  };

  const handleDelete = async (id) => {
    await deleteCategory(id);
    fetchCategories();
  };

  const handleEdit = (category) => {
    setIsEditing(true);
    setCurrentCategory(category);
  };

  const handleCancelEdit = () => {
    setIsEditing(false);
    setCurrentCategory({ id: '', name: '', description: '' });
  };

  const handleChange = (event) => {
    const { name, value } = event.target;
    setCurrentCategory((prevCategory) => ({
      ...prevCategory,
      [name]: value,
    }));
  };

  const handleUpdate = async (event) => {
    event.preventDefault();
    await updateCategory(currentCategory.id, {
      name: currentCategory.name,
      description: currentCategory.description,
    });
    setIsEditing(false);
    setCurrentCategory({ id: '', name: '', description: '' });
    fetchCategories();
  };

  return (
    <div>
      <h2>Categories</h2>
      {isEditing ? (
        <form onSubmit={handleUpdate}>
          <input
            type="text"
            name="name"
            value={currentCategory.name}
            onChange={handleChange}
            placeholder="Category Name"
          />
          <input
            type="text"
            name="description"
            value={currentCategory.description}
            onChange={handleChange}
            placeholder="Category Description"
          />
          <button type="submit" className="button button-update">
            Update
          </button>
          <button type="button" className="button button-delete" onClick={handleCancelEdit}>
            Cancel
          </button>
        </form>
      ) : (
        <CategoryFormFunction fetchCategories={fetchCategories} />
      )}
      <table>
        <thead>
          <tr>
            <th>ID</th>
            <th>Name</th>
            <th>Description</th>
            <th>Actions</th>
          </tr>
        </thead>
        <tbody>
          {categories.map((category) => (
            <tr key={category.id}>
              <td>{category.id}</td>
              <td>{category.name}</td>
              <td>{category.description}</td>
              <td>
                <button className="button button-update" onClick={() => handleEdit(category)}>
                  Edit
                </button>
                <button className="button button-delete" onClick={() => handleDelete(category.id)}>
                  Delete
                </button>
              </td>
            </tr>
          ))}
        </tbody>
      </table>
    </div>
  );
};

export default CategoryListFunction;
\`\`\`

### 6. Lunch Break (30 minutes)

### 7. Adding CSS (30 minutes)
- Demonstrate how to add and apply CSS in React.
- Style the category and product forms.

\`\`\`css
/* Category.css */
form {
  margin: 20px;
}

input {
  margin-right: 10px;
}
\`\`\`

### 8. Integrating MockAPI for CRUD Operations (60 minutes)
- Set up MockAPI for category and product management.
- Perform CRUD operations (Create, Read, Update, Delete) using MockAPI.

\`\`\`jsx
// ApiCategory.js
const apiUrl = 'https://mockapi.io/endpoint';

export const getCategories = () => fetch(apiUrl).then((res) => res.json());
export const createCategory = (category) =>
  fetch(apiUrl, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(category),
  }).then((res) => res.json());

export const deleteCategory = (id) =>
  fetch(\`\${apiUrl}/\${id}\`, {
    method: 'DELETE',
  }).then((res) => res.json());

export const updateCategory = (id, category) =>
  fetch(\`\${apiUrl}/\${id}\`, {
    method: 'PUT',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(category),
  }).then((res) => res.json());
\`\`\`

### 9. Building Pages for Category and Product Registration (60 minutes)
- Create forms for category and product registration.
- Link forms to MockAPI for data submission.

\`\`\`jsx
// Example form structure with API integration
const handleSubmit = (event) => {
  event.preventDefault();
  createCategory({ name: categoryName, description }).then((data) => console.log(data));
};

return (
  <form onSubmit={handleSubmit}>
    <input type="text" placeholder="Enter category name" />
    <button type="submit">Submit</button>
  </form>
);
\`\`\`

- Implement similar logic for \`ProductForm.js\`.

### 10. Building Lists to Display Data (45 minutes)
- Create lists to display categories and products.
- Fetch data from MockAPI and render it in the components.

\`\`\`jsx
// CategoryListFunction.js
import React, { useEffect, useState } from 'react';
import { getCategories } from '../services/ApiCategory';

const CategoryListFunction = () => {
  const [categories, setCategories] = useState([]);

  useEffect(() => {
    getCategories().then((data) => setCategories(data));
  }, []);

  return (
    <div>
      <h2>Category List</h2>
      <ul>
        {categories.map((category) => (
          <li key={category.id}>{category.name}</li>
        ))}
      </ul>
    </div>
  );
};

export default CategoryListFunction;
\`\`\`

- Implement similar logic for \`ProductList.js\`.


### 11. Routing Navigation (15 minutes)
- Modify \`App.js\` to include a welcome message and links to the category and product registration pages.

#### Step-by-Step Instructions:

1. **Install React Router:**
   - Run the following command to install React Router:
   \`\`\`bash
   npm install react-router-dom
   \`\`\`

2. **Modify App.js:**
   - Replace the content of \`App.js\` with the following code:
   \`\`\`jsx
   import React from 'react';
   import { BrowserRouter as Router, Route, Link } from 'react-router-dom';
   import logo from './logo.svg';
   import './App.css';
   import CategoryListClass from './components/Category/CategoryListClass';
   import CategoryFormClass from './components/Category/CategoryFormClass';
   import CategoryListFunction from './components/Category/CategoryListFunction';

   function App() {
     return (
       <Router>
         <div className="App">
           <h1>Product Management System</h1>
           <nav>
             <ul>
               <li>
                 <Link to="/category">Register Category</Link>
               </li>
               <li>
                 <Link to="/product">Register Product</Link>
               </li>
             </ul>
           </nav>
           <Route path="/category" component={CategoryFormClass} />
           <Route path="/product" component={CategoryListFunction} />
         </div>
       </Router>
     );
   }

   export default App;
   \`\`\`

   
### 13. Design Content Navigation (15 minutes)
- Modify \`App.js\` to include a welcome message and links to the category and product registration pages.

#### Step-by-Step Instructions:


1. **Modify App.js:**
   - Replace the content of \`App.js\` with the following code:
   \`\`\`jsx
   import React from 'react';
   import { BrowserRouter as Router, Route, Link } from 'react-router-dom';
   import logo from './logo.svg';
   import './App.css';
   import CategoryListClass from './components/Category/CategoryListClass';
   import CategoryFormClass from './components/Category/CategoryFormClass';
   import CategoryListFunction from './components/Category/CategoryListFunction';

   function App() {
    return (
        <Router>
        <div className="App">
            <h1>Product Management System</h1>
            <div className="container">
            <nav className="sidebar">
                <ul>
                <li>
                    <Link to="/category">Register Category</Link>
                </li>
                <li>
                    <Link to="/product">Register Product</Link>
                </li>
                </ul>
            </nav>
            <div className="content">
                <Routes>
                <Route path="/category" element={<CategoryListClass />} />
                <Route path="/product" element={<CategoryListFunction />} />
                </Routes>
            </div>
            </div>
        </div>
        </Router>
    );
    }

    export default App;
   \`\`\`

2. **Modify App.css:**
   - Update to add the content of \`App.css\` with the following code:
   \`\`\`css
   .App {
    font-family: sans-serif;
    text-align: center;
    }

    .container {
    display: flex;
    height: 100vh;
    }

    .sidebar {
    width: 200px;
    background-color: #f8f9fa;
    padding: 15px;
    box-shadow: 2px 0 5px rgba(0, 0, 0, 0.1);
    }

    .sidebar ul {
    list-style-type: none;
    padding: 0;
    }

    .sidebar ul li {
    margin: 10px 0;
    }

    .sidebar ul li a {
    text-decoration: none;
    color: #007bff;
    font-weight: bold;
    }

    .content {
    flex-grow: 1;
    padding: 20px;
    }

    h1 {
    margin: 20px;
    }
   \`\`\`



## Conclusion (15 minutes)
- Recap the key concepts covered.
- Q&A session to address any doubts or questions.
- Provide resources for further learning.

## Resources:
- [React Documentation](https://reactjs.org/docs/getting-started.html)
- [MockAPI Documentation](https://mockapi.io/docs)
- [CSS Tricks](https://css-tricks.com/)
- [React Router Documentation](https://reactrouter.com/web/guides/quick-start)
- [Redux Documentation](https://redux.js.org/)

### Advanced Topics
- Discuss state management with Context API.
- 3rd-Party libary like axios,sweertalert,formik,bootstrap/tailwind
- Implementing more complex forms and validation.
- Error handling and debugging tips.

This comprehensive 5-hour training module ensures that students gain an in-depth understanding of React components, styling, and CRUD operations, with ample time for hands-on practice and advanced topics.

